% Exact Monte Carlo sampling
%
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%     Markovian Dynamics on Reaction Networks        %%%%%%%%%%%%
%%%%%%%%%%%           Example: Neural dynamics                 %%%%%%%%%%%%
%%%%%%%%%%%           EXACT MONTE CARLO SAMPLING               %%%%%%%%%%%%
%%%%%%%%%%%          Authors: Jenkinson/Goutsias               %%%%%%%%%%%%
%%%%%%%%%%%           Last Modified: 04/20/2012                %%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

clear all;
close all;

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Initialize model parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

parameterSet = 1;   % (1) model with no avalanching
                    % (2) model with avalanching

L     = 100;        % number of neurons
gamma = 0.1;        % neuron decay rate
h     = 0.001;      % external input

if parameterSet==1
    nu_E_plus_nu_I  = 0.004;
    nu_E_minus_nu_I = 0.00276;
else
    nu_E_plus_nu_I  = 0.004;
    nu_E_minus_nu_I = 0.276;
end

nu_E = 0.5*(nu_E_plus_nu_I + nu_E_minus_nu_I); % synaptic excitatory weight
nu_I = 0.5*(nu_E_plus_nu_I - nu_E_minus_nu_I); % synaptic inhibitory weight

M         = 4;      % number of reactions
N         = 2;      % number of species
totalTime = 1000;   % simulation time in ms
memory    = 100000; % preallocated array size

yinit     = [0 0]';
timeStep  = 0.1;
time      = 0:timeStep:(totalTime+1-timeStep);

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%% MONTE CARLO ITERATIONS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

t    = 0;
tt   = repmat(0,1,memory);
Z    = repmat(0,M,1);
Y    = repmat(0,N,1);
YY   = repmat(0,N,memory);
iter = 1;

while t < totalTime
    iter = iter+1;
    phi = nu_E*Y(1)+nu_I*Y(2)+h; % synaptic input
    
    %
    % Propensity functions
    %
    propensity(1) = (L/2-Y(1))*max(tanh(phi),0);
    propensity(2) = gamma*Y(1);
    propensity(3) = (L/2-Y(2))*max(tanh(phi),0);
    propensity(4) = gamma*Y(2);

    %
    % Update state
    %
    astr = sum(propensity);
    if ~astr
        t = totalTime;
    else
        tau  = exprnd(inv(astr));
        t    = t+tau;
        m    = find(cumsum(propensity) > astr*rand,1);
        Z(m) = Z(m)+1;
    end
    Y(1) = yinit(1) + Z(1) - Z(2);
    Y(2) = yinit(2) + Z(3) - Z(4);
    YY(:,iter) = Y;
    tt(iter)   = t;
end

tt(iter) = totalTime;

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%% MONTE CARLO ITERATIONS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

lnw = 1.5;
fig = figure;
stairs(tt(1:iter),YY(1,1:iter)+YY(2,1:iter),'b','LineWidth',lnw)
axis([0 totalTime 0 L])
xlabel('time (msec)')
ylabel('total number of active neurons')

if parameterSet==1
    saveas(fig,'results/spikingNeuronsNoAval.fig')
else
    saveas(fig,'results/spikingNeuronsAval.fig')
end