%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%     Markovian Dynamics on Reaction Networks        %%%%%%%%%%%%
%%%%%%%%%%%           Example: Opinion formation               %%%%%%%%%%%%
%%%%%%%%%%%        EXACT MONTE CARLO SAMPLING METHOD           %%%%%%%%%%%%
%%%%%%%%%%%           Authors: Jenkinson/Goutsias              %%%%%%%%%%%%
%%%%%%%%%%%            Last Modified: 04/18/2012               %%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

clear all;
close all;

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Initialize model parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

parameterSet=1; % 1 is for monostable system (liberal system)
                % 2 is for bistable system (totalitarian system)

if parameterSet==1 % monostable system
    L      = 40;
    kappa1 = 20/L;
    kappa2 = 40/L;
    a1     = 0/L;
    a2     = .5/L;
    a3     = .5/L;
else % bistable system
    L      = 40;
    kappa1 = 20/L;
    kappa2 = 40/L;
    a1     = 1.5/L;
    a2     = 1/L;
    a3     = -.25/L;
end

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%% INPUT PARAMETERS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

mciter    =  4000;                     % NUMBER OF MONTE CARLO ITERATIONS
M         =  4;                        % NUMBER OF REACTIONS
N         =  2;                        % NUMBER OF SPIECES
totalTime =  15;                       % SIMULATION TIME (IN DAYS)

xinit    = [0 0]';
timeStep = .1;
time     = 0:timeStep:(totalTime+1-timeStep);

memory = 1e6;
if parameterSet == 1
    fidz = fopen('results/averagez_monoStable.dat','w');
    fidx = fopen('results/averagex_monoStable.dat','w');
else
    fidz = fopen('results/averagez_biStable.dat','w');
    fidx = fopen('results/averagex_biStable.dat','w');
    
end

display('Running exact Monte Carlo sampling. Please wait...');
tic;

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%% MONTE CARLO ITERATIONS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

for i=1:mciter
    rand(1);
	t     = 0;
	tt    = 0;
    Z     = repmat(0,M,1);
    ZZ    = repmat(0,M,memory);
    X     = repmat(0,N,memory);
    iter  = 1;
    while t < totalTime
        iter = iter+1;
        alpha(1)  = kappa1*(L-(xinit(1)+Z(1)-Z(2)))*...
                      exp(a1*(xinit(1)+Z(1)-Z(2))+a2*(xinit(2)+Z(3)-Z(4)));
        alpha(2)  = kappa1*(L+(xinit(1)+Z(1)-Z(2)))*...
                      exp(-a1*(xinit(1)+Z(1)-Z(2))-a2*(xinit(2)+Z(3)-Z(4)));
        alpha(3)  = kappa2*(L-(xinit(2)+Z(3)-Z(4)))*exp(a3*(xinit(1)+Z(1)-Z(2)));
        alpha(4)  = kappa2*(L+(xinit(2)+Z(3)-Z(4)))*exp(-a3*(xinit(1)+Z(1)-Z(2)));
        astr  = sum(alpha);
        if ~astr
            t = totalTime;
        else
            tau   = exprnd(inv(astr));
            t     = t + tau;
            m     = find(cumsum(alpha) > astr*rand, 1 );
            Z(m)  = Z(m) + 1;
        end
        tt(iter) = t;
        ZZ(:,iter) = Z;
    end 
    tt(end) = totalTime;
    ZZZ = ZZ(:,1:iter);
    ttt = tt(1:iter);
    X(1,1:iter) = xinit(1)+ZZZ(1,1:iter)-ZZZ(2,1:iter); 
    X(2,1:iter) = xinit(2)+ZZZ(3,1:iter)-ZZZ(4,1:iter); 
    XX = X(:,1:iter);
    B1  = interp1(ttt,ZZZ(1,:),time,'nearest');
    B2  = interp1(ttt,ZZZ(2,:),time,'nearest');
    B3  = interp1(ttt,ZZZ(3,:),time,'nearest');
    B4  = interp1(ttt,ZZZ(4,:),time,'nearest');
    fwrite(fidz,[B1;B2;B3;B4],'double');
    C1 = interp1(ttt,XX(1,:),time,'nearest');
    C2 = interp1(ttt,XX(2,:),time,'nearest');
    fwrite(fidx,[C1;C2],'double');
end
toc
fclose(fidz);
fclose(fidx);

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%% MEAN/STD/PMF X COMPUTATIONS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

stime = size(time);

Xdata        = repmat(0,[mciter,N,stime(2)]);

if parameterSet == 1
    fidx = fopen('results/averagex_monoStable.dat','r');
else
    fidx = fopen('results/averagex_biStable.dat','r');
end

for i=1:mciter
    Xdata(i,:,:)=fread(fidx,[N,stime(2)],'double');
end
fclose(fidx);

expectedmean = squeeze(mean(Xdata,1));
expectedstd  = squeeze(std(Xdata,1));

lnw=1.5;

figure, plot(time,expectedmean(1,:),'-','Color','red','LineWidth',lnw);
hold on
plot(time,expectedmean(1,:)+expectedstd(1,:),':','LineWidth',lnw);
hold on
plot(time,expectedmean(1,:)-expectedstd(1,:),':','LineWidth',lnw);
hold on
axis([0 totalTime -40 40]);
title('\bf{exact Monte Carlo sampling}');
xlabel('time (days)');
ylabel('net public opinion');
if parameterSet == 1
    saveas(gcf,'results/EmcMeanPub_monoStable.fig');
else
    saveas(gcf,'results/EmcMeanPub_biStable.fig');
end

figure, plot(time,expectedmean(2,:),'-','Color','red','LineWidth',lnw);
hold on
plot(time,expectedmean(2,:)+expectedstd(2,:),':','LineWidth',lnw);
hold on
plot(time,expectedmean(2,:)-expectedstd(2,:),':','LineWidth',lnw);
hold on
axis([0 totalTime -40 40]);
title('\bf{exact Monte Carlo sampling}');
xlabel('time (days)');
ylabel('net private opinion');
if parameterSet == 1
    saveas(gcf,'results/EmcMeanPriv_monoStable.fig');
else
    saveas(gcf,'results/EmcMeanPriv_biStable.fig');
end

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%% PLOT HISTOGRAMS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

histTime=15;

s1max=max(Xdata(:,1,histTime/timeStep));
s1min=min(Xdata(:,1,histTime/timeStep));
Prob1 = hist(squeeze(Xdata(:,1,histTime/timeStep)),s1min:s1max);
Probapprox1 = Prob1/sum(Prob1);
figure, bar(s1min:s1max,Probapprox1)
colormap white
axis([-L L 0 1.1*max(max(Probapprox1))])
title('\bf{exact Monte Carlo sampling}');
xlabel('net public opinion');
ylabel('probability');
if parameterSet == 1
    saveas(gcf,'results/EmcMargPub_monoStable.fig');
else
    saveas(gcf,'results/EmcMargPub_biStable.fig');
end

s2max=max(Xdata(:,2,histTime/timeStep));
s2min=min(Xdata(:,2,histTime/timeStep));
Prob2 = hist(squeeze(Xdata(:,2,histTime/timeStep)),s2min:s2max);
Probapprox2 = Prob2/sum(Prob2);
figure, bar(s2min:s2max,Probapprox2)
colormap white
axis([-L L 0 1.1*max(max(Probapprox2))])
title('\bf{exact Monte Carlo sampling}');
xlabel('net private opinion');
ylabel('probability');
if parameterSet == 1
    saveas(gcf,'results/EmcMargPriv_monoStable.fig');
else
    saveas(gcf,'results/EmcMargPriv_biStable.fig');
end