%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%     Markovian Dynamics on Reaction Networks        %%%%%%%%%%%%
%%%%%%%%%%%           Example: Opinion formation               %%%%%%%%%%%%
%%%%%%%%%%%        LINEAR NOISE APPROXIMATION METHOD           %%%%%%%%%%%%
%%%%%%%%%%%            Authors: Jenkinson/Goutsias             %%%%%%%%%%%%
%%%%%%%%%%%             Last Modified: 04/18/2012              %%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

clear all;
close all;

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Solve monostable system
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

L     = 40;
OMEGA = L;

kappa1 = 0.5;
kappa2 = 1;
a1     = 0;
a2     = 1/80;
a3     = 1/80;

kappat1 = OMEGA*kappa1;
kappat2 = OMEGA*kappa2;
at1     = OMEGA*a1;
at2     = OMEGA*a2;
at3     = OMEGA*a3;

%
% Compute macroscopic trajectory
%
tf=15;
tspan=0:.01:tf;                  % defines the time points for integration
zeta0=[0; 0; 0; 0];              % defines initial conditions
[time_zeta, zeta] = ode23s(@(t,zeta) socialRHS(t,zeta,kappat1,kappat2,...
                           at1,at2,at3),tspan,zeta0);
                       
%
% Compute covariance matrix
%
C0    = zeros(4,4);
C0vec = reshape(C0,16,1);
display('Running linear noise approximation. Please wait...');
tic;
[time_C,Cvec] = ode23s(@(t,Cvec) socialLYAP(t,Cvec,time_zeta,zeta,...
                 kappat1,kappat2,at1,at2,at3),tspan,C0vec);
             
%
% Convert to population process with Omega=2L=80
%
stoich = [1 -1 0 0;0 0 1 -1];
chi0 = [0;0];
chi  = zeros(length(time_C),2);
chiCovar = zeros(length(time_C),2,2);

for index=1:length(time_C)
    C=reshape(Cvec(index,:),4,4);
    chi(index,:) = OMEGA*(chi0 + stoich*(zeta(index,:)'));
    chiCovar(index,:,:) = OMEGA*stoich*C*stoich';
end

toc

%
% Plot SS joint dist for Omega=2L=80
%
CovarMat = squeeze(chiCovar(end,:,:));
CovarMat = (CovarMat+CovarMat')/2; % symmetrize to remove numerical errors

muVect = [chi(end,1);chi(end,2)];

fig = figure('Position',[50 50 650 700]);
h1 = subplot(1,1,1);

x1 = -L:1:L; x2 = -L:1:L;
[X1,X2] = meshgrid(x1,x2);
for index1=1:2*L+1
    for index2=1:2*L+1
        xVect = [X1(index1,index2);X2(index1,index2)];
        normalPDF(index1,index2) = (1/(2*pi*sqrt(det(CovarMat))))...
            *exp(-.5*(xVect-muVect)'*inv(CovarMat)*(xVect-muVect));
    end
end
normalPDF = reshape(normalPDF,length(x2),length(x1));
surf(x1,x2,normalPDF);
set(h1,'XGrid','on');
set(h1,'YGrid','on');
set(h1,'ZGrid','off');
colormap('jet')
shading('interp')
axis('tight')
title('\bf{linear noise approximation}');
xlabel('net private opinion');
ylabel('net public opinion');
zlabel('probability');
saveas(h1,'results/LnaJoint_monoStable.fig')

lnw=1.5;

%
% Plot SS marginal dist for Omega=2L=80
%
figure;
bar(x1,sum(normalPDF,2));
colormap white
axis([-L L 0 1.1*max(sum(normalPDF,2))])
title('\bf{linear noise approximation}');
xlabel('net public opinion')
ylabel('probability')
saveas(gcf,'results/LnaMargPub_monoStable.fig')

figure;
bar(x1,sum(normalPDF,1));
colormap white
axis([-L L 0 1.1*max(sum(normalPDF,1))])
title('\bf{linear noise approximation}');
xlabel('net private opinion')
ylabel('probability')
saveas(gcf,'results/LnaMargPriv_monoStable.fig')