%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%     Markovian Dynamics on Reaction Networks        %%%%%%%%%%%%
%%%%%%%%%%%               OPINION FORMATION                    %%%%%%%%%%%%
%%%%%%%%%%%           MOMENT APPROXIMATION METHOD              %%%%%%%%%%%%
%%%%%%%%%%%           Authors: Jenkinson/Goutsias              %%%%%%%%%%%%
%%%%%%%%%%%            Last Modified: 04/18/2012               %%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

clear all;
close all;

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Initialize model parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

parameterSet=1; % 1 is for monostable system (liberal democracy)
                % 2 is for bistable system (totalitarian state)

    
if parameterSet==1 % monostable system
    L      = 40;
    kappa1 = 20/L;
    kappa2 = 40/L;
    a1     = 0/L;
    a2     = .5/L;
    a3     = .5/L;
else % bistable system
    L      = 40;
    kappa1 = 20/L;
    kappa2 = 40/L;
    a1     = 1.5/L;
    a2     = 1/L;
    a3     = -.25/L;
end

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%% Simulation without Jensen's inequality correction
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

M         =  4;                        % NUMBER OF REACTIONS
N         =  2;                        % NUMBER OF SPIECES
totalTime =  15;                       % SIMULATION TIME (IN DAYS)

useJensensINEQ = 0;

tspan = 0:.01:totalTime;  % defines the time points for integration
u_c0  = zeros(4+4*4,1);   % set initial condition 

%
% solves mean and covar ODEs
%

tic;
display('Running moment approximation method. Please wait...');

[time, u_c]=ode23s(@(t,u_c) socialRHS(u_c,kappa1,kappa2,a1,a2,a3,L,...
                   useJensensINEQ),tspan,u_c0); 
toc

%
% Convert to population process
%
meanPop = zeros(length(time),2);
covPop  = zeros(length(time),2,2);
stoich  = [1 -1 0 0;0 0 1 -1];
for index = 1:length(time)
    meanPop(index,:) = u_c(index,1:4)*stoich';
    covPop(index,:,:) =  stoich*reshape(u_c(index,5:end),4,4)*stoich';
end

%
% Plot results without Jensen's Inequality correction
%

lnw=1.5;

figure,plot(time,meanPop(:,1),'r','LineWidth',lnw); hold on;
plot(time,meanPop(:,1)+sqrt(covPop(:,1,1)),':','LineWidth',lnw)
plot(time,meanPop(:,1)-sqrt(covPop(:,1,1)),':','LineWidth',lnw)
xlabel('time (days)');ylabel('net public opinion');
title('{\bf moment approximation method}')
axis([0 max(time) -40 40])
if parameterSet == 1
    saveas(gcf,'results/MaMeanPub_monoStable.fig');
else
    saveas(gcf,'results/MaMeanPub_biStable.fig');
end

figure,plot(time,meanPop(:,2),'r','LineWidth',lnw);hold on;
plot(time,meanPop(:,2)+sqrt(covPop(:,2,2)),':','LineWidth',lnw)
plot(time,meanPop(:,2)-sqrt(covPop(:,2,2)),':','LineWidth',lnw)
xlabel('time (days)');ylabel('net private opinion');
title('{\bf moment approximation method}')
axis([0 max(time) -40 40])
if parameterSet == 1
    saveas(gcf,'results/MaMeanPriv_monoStable.fig');
else
    saveas(gcf,'results/MaMeanPriv_biStable.fig');
end

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%% Run simulation with Jensen's inequality correction
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

useJensensINEQ = 1;
                   
%
% solves mean and covar ODEs
%

[time, u_c]=ode23s(@(t,u_c) socialRHS(u_c,kappa1,kappa2,a1,a2,a3,L,...
                 useJensensINEQ),tspan,u_c0); 

%
% Convert to population process
%

meanPop = zeros(length(time),2);
covPop  = zeros(length(time),2,2);
stoich  = [1 -1 0 0;0 0 1 -1];
for index = 1:length(time)
    meanPop(index,:) = u_c(index,1:4)*stoich';
    covPop(index,:,:) =  stoich*reshape(u_c(index,5:end),4,4)*stoich';
end

%
% Plot results with Jensen's inequality correction
%

figure,plot(time,meanPop(:,1),'r','LineWidth',lnw);hold on;
plot(time,meanPop(:,1)+sqrt(covPop(:,1,1)),':','LineWidth',lnw)
plot(time,meanPop(:,1)-sqrt(covPop(:,1,1)),':','LineWidth',lnw)
xlabel('time (days)');ylabel('net public opinion');
title('{\bf moment approximation method (corrected)}')
axis([0 max(time) -40 40])
if parameterSet == 1
    saveas(gcf,'results/MaMeanPub_corr_monoStable.fig');
else
    saveas(gcf,'results/MaMeanPub_corr_biStable.fig');
end

figure,plot(time,meanPop(:,2),'r','LineWidth',lnw);hold on;
plot(time,meanPop(:,2)+sqrt(covPop(:,2,2)),':','LineWidth',lnw)
plot(time,meanPop(:,2)-sqrt(covPop(:,2,2)),':','LineWidth',lnw)
xlabel('time (days)');ylabel('net private opinion');
title('{\bf moment approximation method (corrected)}')
axis([0 max(time) -40 40])
if parameterSet == 1
    saveas(gcf,'results/MaMeanPriv_corr_monoStable.fig');
else
    saveas(gcf,'results/MaMeanPriv_corr_biStable.fig');
end