%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%     Markovian Dynamics on Reaction Networks        %%%%%%%%%%%%
%%%%%%%%%%%           Example: Opinion formation               %%%%%%%%%%%%
%%%%%%%%%%%          POISSON APPROXIMATION METHOD              %%%%%%%%%%%%
%%%%%%%%%%%          Authors: Jenkinson/Goutsias               %%%%%%%%%%%%
%%%%%%%%%%%           Last Modified: 04/18/2012                %%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

clear all;
close all;

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Initialize model parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

parameterSet=1; % 1 is for monostable system (liberal democracy)
                % 2 is for bistable system (totalitarian state)

if parameterSet==1 % monostable system
    L      = 40;
    kappa1 = 20/L;
    kappa2 = 40/L;
    a1     = 0/L;
    a2     = .5/L;
    a3     = .5/L;
else % bistable system
    L      = 40;
    kappa1 = 20/L;
    kappa2 = 40/L;
    a1     = 1.5/L;
    a2     = 1/L;
    a3     = -.25/L;
end

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%% INPUT PARAMETERS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

mciter    =  4000;                  % NUMBER OF MONTE CARLO ITERATIONS
M         =  4;                     % NUMBER OF REACTIONS
N         =  2;                     % NUMBER OF SPIECES
totalTime =  15;                    % SIMULATION TIME (IN DAYS)
dt        =  0.1;                   % SAMPLING RATE (IN DAYS)

xinit = [0 0]';

if parameterSet == 1
    fidz = fopen('results/averagez_monoStable.dat','w');
else
    fidz = fopen('results/averagez_biStable.dat','w');
end

display('Running Poisson approximation method. Please wait...');
tic;
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%% MONTE CARLO ITERATIONS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

for i=1:mciter
    rand(1);
    memory = 1e6;
    Z   = repmat(0,M,1);
    ZZ  = repmat(0,M,memory);
    iter = 1;
    for t=dt:dt:totalTime
        iter  = iter + 1;
        alpha(1)  = kappa1*(L-(xinit(1)+Z(1)-Z(2)))*...
                      exp(a1*(xinit(1)+Z(1)-Z(2))+a2*(xinit(2)+Z(3)-Z(4)));
        alpha(2)  = kappa1*(L+(xinit(1)+Z(1)-Z(2)))*...
                      exp(-a1*(xinit(1)+Z(1)-Z(2))-a2*(xinit(2)+Z(3)-Z(4)));
        alpha(3)  = kappa2*(L-(xinit(2)+Z(3)-Z(4)))*exp(a3*(xinit(1)+Z(1)-Z(2)));
        alpha(4)  = kappa2*(L+(xinit(2)+Z(3)-Z(4)))*exp(-a3*(xinit(1)+Z(1)-Z(2)));

        alpha(alpha<0) = 0;  
       
        R          = poissrnd(alpha.*dt);
        Z          = Z + R';
        ZZ(:,iter) = Z;
    end 
    ZZZ = ZZ(:,1:iter);
    B   = ZZZ(:,1:1:totalTime/dt);
    fwrite(fidz,B,'double');
end

toc
fclose(fidz);

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%% MEAN/STD/PMF COMPUTATIONS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

Zdata  = repmat(0,[mciter,M,totalTime/(1*dt)]);
Xdata  = repmat(0,[mciter,N,totalTime/(1*dt)]);

if parameterSet == 1
    fidz = fopen('results/averagez_monoStable.dat','r');
else
    fidz = fopen('results/averagez_biStable.dat','r');
end

for i=1:mciter
    Zdata(i,:,:)=fread(fidz,[M,totalTime/(1*dt)],'double');
end
fclose(fidz);

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%% MAKE PLOTS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

for i=1:mciter
    Xdata(i,1,1:totalTime/(1*dt))=xinit(1)+Zdata(i,1,1:totalTime/(1*dt))...
                                          -Zdata(i,2,1:totalTime/(1*dt));
    Xdata(i,2,1:totalTime/(1*dt))=xinit(2)+Zdata(i,3,1:totalTime/(1*dt))...
                                          -Zdata(i,4,1:totalTime/(1*dt));  
end

clear expectedmean expectedstd
expectedmean = squeeze(mean(Xdata,1));
expectedstd  = squeeze(std(Xdata,1));

lnw=1.5;
points  = totalTime;
times   = 0:1*dt:totalTime;
time    = times(1:totalTime/(1*dt));

figure, plot(time,expectedmean(1,:),'-','Color','red','LineWidth',lnw);
hold on
plot(time,expectedmean(1,:)+expectedstd(1,:),':','LineWidth',lnw);
hold on
plot(time,expectedmean(1,:)-expectedstd(1,:),':','LineWidth',lnw);
hold on
axis([0 points -L L]);
title('{\bf Poisson approximation method}');
xlabel('time (days)');
ylabel('net public opinion');
if parameterSet == 1
    saveas(gcf,'results/PoissonMeanPub_monoStable.fig');
else
    saveas(gcf,'results/PoissonMeanPub_biStable.fig');
end

figure, plot(time,expectedmean(2,:),'-','Color','red','LineWidth',lnw);
hold on
plot(time,expectedmean(2,:)+expectedstd(2,:),':','LineWidth',lnw);
hold on
plot(time,expectedmean(2,:)-expectedstd(2,:),':','LineWidth',lnw);
hold on
axis([0 points -L L]);
title('{\bf Poisson approximation method}');
xlabel('time (days)');
ylabel('net private opinion');
if parameterSet == 1
    saveas(gcf,'results/PoissonMeanPriv_monoStable.fig');
else
    saveas(gcf,'results/PoissonMeanPriv_biStable.fig');
end

histTime = 15;

s1max = max(Xdata(:,1,histTime/(dt)));
s1min = min(Xdata(:,1,histTime/(dt)));
Prob1 = hist(squeeze(Xdata(:,1,histTime/(dt))),s1min:s1max);
Probapprox1 = Prob1/sum(Prob1);
figure, bar(s1min:s1max,Probapprox1)
colormap white
axis([-L L 0 1.1*max(max(Probapprox1))])
title('{\bf Poisson approximation method}');
xlabel('net public opinion');
ylabel('probability');
if parameterSet == 1
    saveas(gcf,'results/PoissonMargPub_monoStable.fig');
else
    saveas(gcf,'results/PoissonMargPub_biStable.fig');
end

s2max=max(Xdata(:,2,histTime/(dt)));
s2min=min(Xdata(:,2,histTime/(dt)));
Prob2 = hist(squeeze(Xdata(:,2,histTime/(dt))),s2min:s2max);
Probapprox2 = Prob2/sum(Prob2);
figure, bar(s2min:s2max,Probapprox2)
colormap white
axis([-L L 0 1.1*max(max(Probapprox2))])
title('{\bf Poisson approximation method}');
xlabel('net private opinion');
ylabel('probability');
if parameterSet == 1
    saveas(gcf,'results/PoissonMargPriv_monoStable.fig');
else
    saveas(gcf,'results/PoissonMargPriv_biStable.fig');
end